@@ -12,10 +12,14 @@ import aiohttp.client_exceptions
 from tqdm import tqdm
 
 from cleanlab_tlm.errors import (
+    HTTP_BAD_REQUEST,
     HTTP_OK,
     HTTP_TOO_MANY_REQUESTS,
+    HTTP_UNAUTHORIZED,
     HTTP_UNPROCESSABLE_ENTITY,
     APIError,
+    AuthError,
+    HTTPBadRequestError,
     InvalidProjectConfigurationError,
     RateLimitError,
     TlmBadRequestError,
@@ -87,6 +91,21 @@ def handle_api_error_from_json(res_json: JSONDict, status_code: Optional[int] =
         raise APIError(f"API call failed with status code {status_code}")
 
 
+async def handle_http_bad_request_error_from_resp(resp: aiohttp.ClientResponse) -> None:
+    """Catches 400 (bad request) errors."""
+    if resp.status == HTTP_BAD_REQUEST:
+        res_json = await resp.json()
+        raise HTTPBadRequestError(res_json["error"])
+
+
+async def handle_api_key_error_from_resp(resp: aiohttp.ClientResponse) -> None:
+    """Catches 401 (unauthorized) errors."""
+    if resp.status == HTTP_UNAUTHORIZED:
+        res_json = await resp.json()
+        if res_json.get("code", None) == "invalid_api_key":
+            raise AuthError("Invalid API key. Check https://tlm.cleanlab.ai/ for your current API key.")
+
+
 def handle_rate_limit_error_from_resp(resp: aiohttp.ClientResponse) -> None:
     """Catches 429 (rate limit) errors."""
     if resp.status == HTTP_TOO_MANY_REQUESTS:
@@ -180,7 +199,10 @@ def tlm_retry(func: Callable[..., Any]) -> Callable[..., Any]:
             except TlmBadRequestError:
                 # dont retry for client-side errors
                 raise
-            except Exception as e:
+            except AuthError:
+                # dont retry for auth errors
+                raise
+            except (HTTPBadRequestError, Exception) as e:
                 sleep_time = 2**num_general_retry
                 num_general_retry += 1
                 error_message = str(e)
@@ -248,6 +270,8 @@ async def tlm_prompt(
 
             res_json = await res.json()
 
+            await handle_api_key_error_from_resp(res)
+            await handle_http_bad_request_error_from_resp(res)
             handle_rate_limit_error_from_resp(res)
             await handle_tlm_client_error_from_resp(res, batch_index)
             await handle_tlm_api_error_from_resp(res, batch_index)
@@ -312,6 +336,8 @@ async def tlm_get_confidence_score(
 
             res_json = await res.json()
 
+            await handle_api_key_error_from_resp(res)
+            await handle_http_bad_request_error_from_resp(res)
             handle_rate_limit_error_from_resp(res)
             await handle_tlm_client_error_from_resp(res, batch_index)
             await handle_tlm_api_error_from_resp(res, batch_index)
@@ -383,6 +409,8 @@ async def tlm_rag_generate(
 
             res_json = await res.json()
 
+            await handle_api_key_error_from_resp(res)
+            await handle_http_bad_request_error_from_resp(res)
             handle_rate_limit_error_from_resp(res)
             await handle_tlm_client_error_from_resp(res, batch_index)
             await handle_tlm_api_error_from_resp(res, batch_index)
@@ -408,7 +436,7 @@ async def tlm_rag_generate(
         if evaluation.name not in [_TLM_RESPONSE_KEY, _TLM_TRUSTWORTHINESS_KEY]:
             ordered_res[evaluation.name] = res_json[evaluation.name]
 
-    return cast(JSONDict, ordered_res)
+    return ordered_res
 
 
 @tlm_retry
@@ -471,6 +499,8 @@ async def tlm_rag_score(
 
             res_json = await res.json()
 
+            await handle_api_key_error_from_resp(res)
+            await handle_http_bad_request_error_from_resp(res)
             handle_rate_limit_error_from_resp(res)
             await handle_tlm_client_error_from_resp(res, batch_index)
             await handle_tlm_api_error_from_resp(res, batch_index)
@@ -496,4 +526,4 @@ async def tlm_rag_score(
         if evaluation.name not in [_TLM_RESPONSE_KEY, _TLM_TRUSTWORTHINESS_KEY]:
             ordered_res[evaluation.name] = res_json[evaluation.name]
 
-    return cast(JSONDict, ordered_res)
+    return ordered_res
