@@ -1,4 +1,5 @@
 import os
+import warnings
 from collections.abc import Sequence
 from typing import Any, Callable, Optional, Union
 
@@ -12,6 +13,7 @@ from cleanlab_tlm.internal.constants import (
     TLM_MODELS_NOT_SUPPORTING_EXPLANATION,
     TLM_NUM_CANDIDATE_RESPONSES_RANGE,
     TLM_NUM_CONSISTENCY_SAMPLES_RANGE,
+    TLM_NUM_SELF_REFLECTIONS_RANGE,
     TLM_REASONING_EFFORT_VALUES,
     TLM_SIMILARITY_MEASURES,
     TLM_TASK_SUPPORTING_CONSTRAIN_OUTPUTS,
@@ -35,16 +37,6 @@ def validate_tlm_prompt(prompt: Union[str, Sequence[str]]) -> None:
         )
 
 
-def validate_tlm_try_prompt(prompt: Sequence[str]) -> None:
-    if isinstance(prompt, str):
-        raise ValidationError("Invalid type str, prompt must be a list/iterable of strings.")
-
-    if isinstance(prompt, Sequence) and any(not isinstance(p, str) for p in prompt):
-        raise ValidationError(
-            "Some items in prompt are of invalid types, all items in the prompt list must be of type str."
-        )
-
-
 def validate_tlm_prompt_response(prompt: Union[str, Sequence[str]], response: Union[str, Sequence[str]]) -> None:
     if isinstance(prompt, str):
         if not isinstance(response, str):
@@ -74,25 +66,11 @@ def validate_tlm_prompt_response(prompt: Union[str, Sequence[str]], response: Un
             )
 
 
-def validate_try_tlm_prompt_response(prompt: Sequence[str], response: Sequence[str]) -> None:
-    if isinstance(prompt, str):
-        raise ValidationError("Invalid type str, prompt must be a list/iterable of strings.")
-
-    if isinstance(prompt, Sequence):
-        if len(prompt) != len(response):
-            raise ValidationError("Length of the prompt and response lists must match.")
-
-        if any(not isinstance(p, str) for p in prompt):
-            raise ValidationError(
-                "Some items in prompt are of invalid types, all items in the prompt list must be of type str."
-            )
-        if any(not isinstance(r, str) for r in response):
-            raise ValidationError(
-                "Some items in response are of invalid types, all items in the response list must be of type str."
-            )
-
-
-def validate_tlm_options(options: Any, support_custom_eval_criteria: bool = True) -> None:
+def validate_tlm_options(
+    options: Any,
+    support_custom_eval_criteria: bool = True,
+    allow_custom_model: bool = False,
+) -> None:
     from cleanlab_tlm.tlm import TLMOptions
 
     if SKIP_VALIDATE_TLM_OPTIONS:
@@ -124,7 +102,7 @@ def validate_tlm_options(options: Any, support_custom_eval_criteria: bool = True
                 )
 
         elif option == "model":
-            if val not in _VALID_TLM_MODELS:
+            if not allow_custom_model and val not in _VALID_TLM_MODELS:
                 raise ValidationError(f"{val} is not a supported model, valid models include: {_VALID_TLM_MODELS}")
 
         elif option == "num_candidate_responses":
@@ -145,10 +123,25 @@ def validate_tlm_options(options: Any, support_custom_eval_criteria: bool = True
                     f"Invalid value {val}, num_consistency_samples must be in the range {TLM_NUM_CONSISTENCY_SAMPLES_RANGE}"
                 )
 
+        elif option == "num_self_reflections":
+            if not isinstance(val, int):
+                raise ValidationError(f"Invalid type {type(val)}, num_self_reflections must be an integer")
+
+            if val < TLM_NUM_SELF_REFLECTIONS_RANGE[0] or val > TLM_NUM_SELF_REFLECTIONS_RANGE[1]:
+                raise ValidationError(
+                    f"Invalid value {val}, num_self_reflections must be in the range {TLM_NUM_SELF_REFLECTIONS_RANGE}"
+                )
+
         elif option == "use_self_reflection":
             if not isinstance(val, bool):
                 raise ValidationError(f"Invalid type {type(val)}, use_self_reflection must be a boolean")
 
+            warnings.warn(
+                "Deprecated method. Use `num_self_reflections` instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
         elif option == "similarity_measure":
             if val not in TLM_SIMILARITY_MEASURES:
                 raise ValidationError(
@@ -183,15 +176,22 @@ def validate_tlm_options(options: Any, support_custom_eval_criteria: bool = True
             if not isinstance(val, list):
                 raise ValidationError(f"Invalid type {type(val)}, custom_eval_criteria must be a list of dictionaries.")
 
+            supported_keys = {"name", "criteria"}
+
             for i, criteria in enumerate(val):
                 if not isinstance(criteria, dict):
                     raise ValidationError(f"Item {i} in custom_eval_criteria is not a dictionary.")
 
-                if "name" not in criteria:
-                    raise ValidationError(f"Missing required key 'name' in custom_eval_criteria item {i}.")
-
-                if "criteria" not in criteria:
-                    raise ValidationError(f"Missing required key 'criteria' in custom_eval_criteria item {i}.")
+                input_keys = set(criteria.keys())
+                if input_keys != supported_keys:
+                    missing = supported_keys - input_keys
+                    extra = input_keys - supported_keys
+                    if missing:
+                        raise ValidationError(f"Missing required keys {missing} in custom_eval_criteria item {i}.")
+                    if extra:
+                        raise ValidationError(
+                            f"Invalid keys {extra} found in custom_eval_criteria item {i}. Supported keys are: {supported_keys}."
+                        )
 
                 if not isinstance(criteria.get("name"), str):
                     raise ValidationError(f"'name' in custom_eval_criteria item {i} must be a string.")
@@ -345,7 +345,7 @@ def validate_rag_inputs(
     form_prompt: Optional[Callable[[str, str], str]] = None,
     response: Optional[Union[str, Sequence[str]]] = None,
     evals: Optional[list[Any]] = None,
-) -> str:
+) -> Union[str, Sequence[str]]:
     """
     Validate inputs for TrustworthyRAG generate and score methods.
 
@@ -354,19 +354,17 @@ def validate_rag_inputs(
     validates parameter types, ensures prompt formatting is handled correctly, and verifies
     that the necessary inputs for any evaluation objects are present.
 
-    Currently, batch processing (sequences of inputs) is not supported and will raise a NotImplementedError.
-
     Args:
-        is_generate: Whether this validation is for the generate method (True) or score method (False).
-        query: The user query or sequence of queries.
-        context: The context used for RAG or sequence of contexts.
-        prompt: Optional pre-formatted prompt string. Cannot be used with form_prompt.
-        form_prompt: Optional function to format a prompt from query and context. Cannot be used with prompt.
-        response: The response to evaluate or sequence of responses (required for score, not for generate).
-        evals: List of evaluation objects to validate against.
+        is_generate (bool): Whether this validation is for a generate call or not.
+        query (str | Sequence[str]): The user query (or multiple queries) to be answered.
+        context (str | Sequence[str]): The context (or contexts) to use when answering the query.
+        prompt (str | Sequence[str], optional): Optional prompt string (or prompts) to use instead of forming a prompt from query and context.
+        form_prompt (Callable[[str, str], str], optional): Optional function to format the prompt based on query and context.
+        response (str | Sequence[str], optional): Optional response (or responses) to score. Required when is_generate=False.
+        evals (list[Any], optional): List of evaluation criteria to use.
 
     Returns:
-        The formatted prompt string if all validation checks pass.
+        str | Sequence[str]: Formatted prompts if all validation checks pass.
 
     Raises:
         ValidationError: If any validation check fails (incompatible parameters, missing required inputs, etc.).
@@ -378,38 +376,84 @@ def validate_rag_inputs(
             "'prompt' and 'form_prompt' cannot be provided at the same time. Use either one, not both."
         )
 
+    # Ensure consistent input formats - either all strings or all lists
+    inputs_to_check = [("query", query), ("context", context), ("prompt", prompt)]
+    if not is_generate:
+        inputs_to_check.append(("response", response))
+
+    # Filter out None values
+    inputs_to_check = [(name, value) for name, value in inputs_to_check if value is not None]
+
+    # Check if any input is a list (not a string)
+    is_any_list = any(isinstance(value, Sequence) and not isinstance(value, str) for _, value in inputs_to_check)
+
+    # Validate format consistency and length matching for list inputs
+    if is_any_list:
+        # All inputs should be lists
+        string_inputs = [(name, value) for name, value in inputs_to_check if isinstance(value, str)]
+        if string_inputs:
+            raise ValidationError(
+                f"Inconsistent input formats: {string_inputs[0][0]} is a string while other inputs are lists. "
+                f"All inputs must be either strings or lists."
+            )
+
+        # All lists should have the same length
+        list_lengths = [
+            (name, len(value))
+            for name, value in inputs_to_check
+            if isinstance(value, Sequence) and not isinstance(value, str)
+        ]
+        if len({length for _, length in list_lengths}) > 1:
+            lengths_str = ", ".join(f"{name}: {length}" for name, length in list_lengths)
+            raise ValidationError(
+                f"Input lists have different lengths: {lengths_str}. " f"All input lists must have the same length."
+            )
+
     # Check for batch inputs - simplified by using a list of parameters to check
     batch_params = [(query, "query"), (context, "context"), (prompt, "prompt")]
     if not is_generate:
         batch_params.append((response, "response"))
 
-    for param_tuple in batch_params:
-        if param_tuple[0] and isinstance(param_tuple[0], Sequence) and not isinstance(param_tuple[0], str):
-            raise NotImplementedError(
-                "Batch processing is not yet supported. It will be available in a future release."
-            )
-
-    # Validate required parameters based on method
-    if is_generate:
-        if query is None or context is None:
-            raise ValidationError("Both 'query' and 'context' are required parameters")
-    else:
-        if response is None:
-            raise ValidationError("'response' is a required parameter")
-        if prompt is None and (query is None or context is None):
-            raise ValidationError("Either 'prompt' or both 'query' and 'context' must be provided")
+    if query is None or context is None:
+        raise ValidationError("Both 'query' and 'context' are required parameters")
+    if not is_generate and response is None:
+        raise ValidationError("'response' is a required parameter")
 
     # Format prompt if needed
-    formatted_prompt = prompt
+    formatted_prompts: Union[str, Sequence[str]] = "" if prompt is None else prompt
     if prompt is None and form_prompt is not None:
-        if query is None or context is None:
-            raise ValidationError("Both 'query' and 'context' are required when using 'form_prompt'")
-        formatted_prompt = form_prompt(str(query), str(context))
+        # Handle both single and batch cases for formatting
+        if isinstance(query, str) and isinstance(context, str):
+            formatted_prompts = form_prompt(query, context)
+        elif (
+            isinstance(query, Sequence)
+            and isinstance(context, Sequence)
+            and not isinstance(query, str)
+            and not isinstance(context, str)
+        ):
+            if len(query) != len(context):
+                raise ValidationError("'query' and 'context' sequences must have the same length for batch processing")
+            formatted_prompts = [form_prompt(q, c) for q, c in zip(query, context)]
+        else:
+            raise ValidationError("'query' and 'context' must both be either strings or sequences of strings")
 
     # Validate parameter types - reuse the batch_params list
     for param_tuple in batch_params:
-        if param_tuple[0] is not None and not isinstance(param_tuple[0], str):
-            raise ValidationError(f"'{param_tuple[1]}' must be a string")
+        param_value, param_name = param_tuple
+        if param_value is not None:
+            if isinstance(param_value, str):
+                # String input is valid
+                pass
+            elif isinstance(param_value, Sequence) and not isinstance(param_value, str):
+                # Check that all items in the sequence are strings
+                if any(not isinstance(item, str) for item in param_value):
+                    raise ValidationError(
+                        f"All items in '{param_name}' must be of type string when providing a sequence"
+                    )
+            else:
+                raise ValidationError(
+                    f"'{param_name}' must be either a string or a sequence of strings, not {type(param_value)}"
+                )
 
     # Validate inputs for evaluations - collect all missing inputs in one error message
     if evals:
@@ -427,4 +471,4 @@ def validate_rag_inputs(
                     f"Missing required input(s) {', '.join(missing_inputs)} for evaluation '{eval_obj.name}'"
                 )
 
-    return str(formatted_prompt)
+    return formatted_prompts
