@@ -13,6 +13,7 @@ This feature is in Beta, contact us if you encounter issues.
 from __future__ import annotations
 
 import asyncio
+from collections.abc import Sequence
 from typing import (
     # lazydocs: ignore
     TYPE_CHECKING,
@@ -23,8 +24,10 @@ from typing import (
     cast,
 )
 
+from tqdm.asyncio import tqdm_asyncio
 from typing_extensions import NotRequired, TypedDict
 
+from cleanlab_tlm.errors import ValidationError
 from cleanlab_tlm.internal.api import api
 from cleanlab_tlm.internal.base import BaseTLM
 from cleanlab_tlm.internal.constants import (
@@ -34,13 +37,17 @@ from cleanlab_tlm.internal.constants import (
     _TLM_EVAL_NAME_KEY,
     _TLM_EVAL_QUERY_IDENTIFIER_KEY,
     _TLM_EVAL_RESPONSE_IDENTIFIER_KEY,
-    _VALID_TLM_QUALITY_PRESETS_RAG,
+    _TLM_MAX_RETRIES,
+    _VALID_TLM_QUALITY_PRESETS,
 )
 from cleanlab_tlm.internal.exception_handling import handle_tlm_exceptions
-from cleanlab_tlm.internal.validation import tlm_score_process_response_and_kwargs, validate_rag_inputs
+from cleanlab_tlm.internal.validation import (
+    tlm_score_process_response_and_kwargs,
+    validate_rag_inputs,
+)
 
 if TYPE_CHECKING:
-    from collections.abc import Sequence
+    from collections.abc import Coroutine
 
     from cleanlab_tlm.internal.types import TLMQualityPreset
     from cleanlab_tlm.tlm import TLMOptions
@@ -60,13 +67,13 @@ class TrustworthyRAG(BaseTLM):
     differences are described below. For details about each argument, refer to the [TLM](../tlm/#class-tlm) documentation.
 
     Args:
-        quality_preset ({"base", "low", "medium"}, default = "medium"): an optional preset configuration to control
+        quality_preset ({"base", "low", "medium", "high", "best"}, default = "medium"): an optional preset configuration to control
             the quality of generated LLM responses and trustworthiness scores vs. latency/costs.
 
         api_key (str, optional): API key for accessing TLM. If not provided, this client will
             attempt to use the CLEANLAB_TLM_API_KEY environment variable.
 
-        options (TLMOptions, optional): a typed dict of advanced configurations you can optionally specify.
+        options ([TLMOptions](../tlm/#class-tlmoptions), optional): a typed dict of advanced configurations you can optionally specify.
             The "custom_eval_criteria" key for [TLM](../tlm/#class-tlm) is not supported for `TrustworthyRAG`, you can instead specify `evals`.
 
         timeout (float, optional): timeout (in seconds) to apply to each request.
@@ -77,6 +84,7 @@ class TrustworthyRAG(BaseTLM):
             If not specified, default evaluations will be used (access these via [get_default_evals](#function-get_default_evals)).
             To come up with your custom `evals`, we recommend you first run [get_default_evals()](#function-get_default_evals) and then add/remove/modify the returned list.
             Each [Eval](#class-eval) in this list provides real-time detection of specific issues in your RAG application based on the user query, retrieved context (documents), and/or LLM-generated response.
+            Set this to an empty list to only score response trustworthiness without additional evaluations.
     """
 
     def __init__(
@@ -95,7 +103,7 @@ class TrustworthyRAG(BaseTLM):
         # Initialize base class
         super().__init__(
             quality_preset=quality_preset,
-            valid_quality_presets=_VALID_TLM_QUALITY_PRESETS_RAG,
+            valid_quality_presets=_VALID_TLM_QUALITY_PRESETS,
             support_custom_eval_criteria=False,
             api_key=api_key,
             options=options,
@@ -117,6 +125,10 @@ class TrustworthyRAG(BaseTLM):
                 for eval_config in _DEFAULT_EVALS
             ]
         else:
+            # validate that evals is a list of Eval objects
+            if not isinstance(evals, list) or any(not isinstance(ev, Eval) for ev in evals):
+                raise ValidationError("'evals' must be a list of Eval objects")
+
             self._evals = evals
 
     def score(
@@ -131,7 +143,70 @@ class TrustworthyRAG(BaseTLM):
         """
         Evaluate an existing RAG system's response to a given user query and retrieved context.
 
-        Batch processing (supplying lists of strings) will be supported shortly in a future release.
+        Args:
+             response (str | Sequence[str]): A response (or list of multiple responses) from your LLM/RAG system.
+             query (str | Sequence[str]): The user query (or list of multiple queries) that was used to generate the response.
+             context (str | Sequence[str]): The context (or list of multiple contexts) that was retrieved from the RAG Knowledge Base and used to generate the response.
+             prompt (str | Sequence[str], optional): Optional prompt (or list of multiple prompts) representing the actual inputs (combining query, context, and system instructions into one string) to the LLM that generated the response.
+             form_prompt (Callable[[str, str], str], optional): Optional function to format the prompt based on query and context. Cannot be provided together with prompt, provide one or the other.
+                    This function should take query and context as parameters and return a formatted prompt string.
+                    If not provided, a default prompt formatter will be used.
+                    To include a system prompt or any other special instructions for your LLM,
+                    incorporate them directly in your custom `form_prompt()` function definition.
+
+        Returns:
+             TrustworthyRAGScore | list[TrustworthyRAGScore]: [TrustworthyRAGScore](#class-trustworthyragscore) object containing evaluation metrics.
+                 If multiple inputs were provided in lists, a list of TrustworthyRAGScore objects is returned, one for each set of inputs.
+        """
+        if prompt is None and form_prompt is None:
+            form_prompt = TrustworthyRAG._default_prompt_formatter
+
+        formatted_prompts = validate_rag_inputs(
+            query=query,
+            context=context,
+            response=response,
+            prompt=prompt,
+            form_prompt=form_prompt,
+            evals=self._evals,
+            is_generate=False,
+        )
+
+        # Support constrain_outputs later
+        processed_responses = tlm_score_process_response_and_kwargs(formatted_prompts, response, None, {})
+
+        # Check if we're handling a batch or a single item
+        if isinstance(query, str) and isinstance(context, str) and isinstance(processed_responses, dict):
+            return self._event_loop.run_until_complete(
+                self._score_async(
+                    response=processed_responses,
+                    prompt=formatted_prompts,
+                    query=query,
+                    context=context,
+                    timeout=self._timeout,
+                )
+            )
+
+        # Batch processing
+        return self._event_loop.run_until_complete(
+            self._batch_score(
+                responses=cast(Sequence[dict[str, Any]], processed_responses),
+                prompts=formatted_prompts,
+                queries=query,
+                contexts=context,
+            )
+        )
+
+    async def score_async(
+        self,
+        *,
+        response: Union[str, Sequence[str]],
+        query: Union[str, Sequence[str]],
+        context: Union[str, Sequence[str]],
+        prompt: Optional[Union[str, Sequence[str]]] = None,
+        form_prompt: Optional[Callable[[str, str], str]] = None,
+    ) -> Union[TrustworthyRAGScore, list[TrustworthyRAGScore]]:
+        """
+        Asynchronously evaluate an existing RAG system's response to a given user query and retrieved context.
 
         Args:
              response (str | Sequence[str]): A response (or list of multiple responses) from your LLM/RAG system.
@@ -146,11 +221,12 @@ class TrustworthyRAG(BaseTLM):
 
         Returns:
              TrustworthyRAGScore | list[TrustworthyRAGScore]: [TrustworthyRAGScore](#class-trustworthyragscore) object containing evaluation metrics.
+                 If multiple inputs were provided in lists, a list of TrustworthyRAGScore objects is returned, one for each set of inputs.
         """
         if prompt is None and form_prompt is None:
             form_prompt = TrustworthyRAG._default_prompt_formatter
 
-        formatted_prompt = validate_rag_inputs(
+        formatted_prompts = validate_rag_inputs(
             query=query,
             context=context,
             response=response,
@@ -161,16 +237,24 @@ class TrustworthyRAG(BaseTLM):
         )
 
         # Support constrain_outputs later
-        processed_response = tlm_score_process_response_and_kwargs(formatted_prompt, response, None, {})
+        processed_responses = tlm_score_process_response_and_kwargs(formatted_prompts, response, None, {})
 
-        return self._event_loop.run_until_complete(
-            self._score_async(
-                response=processed_response,
-                prompt=formatted_prompt,
+        # Check if we're handling a batch or a single item
+        if isinstance(query, str) and isinstance(context, str) and isinstance(processed_responses, dict):
+            return await self._score_async(
+                response=processed_responses,
+                prompt=formatted_prompts,
                 query=query,
                 context=context,
                 timeout=self._timeout,
             )
+
+        # Batch processing
+        return await self._batch_score(
+            responses=cast(Sequence[dict[str, Any]], processed_responses),
+            prompts=formatted_prompts,
+            queries=query,
+            contexts=context,
         )
 
     def generate(
@@ -195,16 +279,32 @@ class TrustworthyRAG(BaseTLM):
         if prompt is None and form_prompt is None:
             form_prompt = TrustworthyRAG._default_prompt_formatter
 
-        formatted_prompt = validate_rag_inputs(
-            query=query, context=context, prompt=prompt, form_prompt=form_prompt, evals=self._evals, is_generate=True
+        formatted_prompts = validate_rag_inputs(
+            query=query,
+            context=context,
+            prompt=prompt,
+            form_prompt=form_prompt,
+            evals=self._evals,
+            is_generate=True,
         )
 
+        # Check if we're handling a batch or a single item
+        if isinstance(query, str) and isinstance(context, str) and isinstance(formatted_prompts, str):
+            return self._event_loop.run_until_complete(
+                self._generate_async(
+                    prompt=formatted_prompts,
+                    query=query,
+                    context=context,
+                    timeout=self._timeout,
+                )
+            )
+
+        # Batch processing
         return self._event_loop.run_until_complete(
-            self._generate_async(
-                prompt=formatted_prompt,
-                query=query,
-                context=context,
-                timeout=self._timeout,
+            self._batch_generate(
+                prompts=formatted_prompts,
+                queries=query,
+                contexts=context,
             )
         )
 
@@ -223,6 +323,113 @@ class TrustworthyRAG(BaseTLM):
         """
         return self._evals.copy()
 
+    async def _batch_generate(
+        self,
+        prompts: Sequence[str],
+        queries: Sequence[str],
+        contexts: Sequence[str],
+    ) -> list[TrustworthyRAGResponse]:
+        """Run a batch of generate operations through TrustworthyRAG. The list returned will have the same length as the input list.
+
+        Args:
+            prompts (Sequence[str]): list of prompts to run
+            queries (Sequence[str]): list of queries corresponding to each prompt
+            contexts (Sequence[str]): list of contexts corresponding to each prompt
+
+        Returns:
+            list[TrustworthyRAGResponse]: TrustworthyRAG responses/scores for each input (in supplied order)
+        """
+
+        # run batch of TrustworthyRAG generate
+        rag_responses = await self._batch_async(
+            [
+                self._generate_async(
+                    prompt=prompt,
+                    query=query,
+                    context=context,
+                    timeout=self._timeout,
+                    capture_exceptions=True,
+                    batch_index=batch_index,
+                )
+                for batch_index, (prompt, query, context) in enumerate(zip(prompts, queries, contexts))
+            ]
+        )
+
+        return cast(list[TrustworthyRAGResponse], rag_responses)
+
+    async def _batch_score(
+        self,
+        responses: Sequence[dict[str, Any]],
+        prompts: Sequence[str],
+        queries: Sequence[str],
+        contexts: Sequence[str],
+    ) -> list[TrustworthyRAGScore]:
+        """Run a batch of score operations through TrustworthyRAG. The list returned will have the same length as the input list.
+
+        Args:
+            responses (Sequence[dict[str, Any]]): list of processed responses to score
+            prompts (Sequence[str]): list of prompts corresponding to each response
+            queries (Sequence[str]): list of queries corresponding to each response
+            contexts (Sequence[str]): list of contexts corresponding to each response
+
+        Returns:
+            list[TrustworthyRAGScore]: TrustworthyRAG scores for each input (in supplied order)
+        """
+
+        # run batch of TrustworthyRAG score
+        rag_scores = await self._batch_async(
+            [
+                self._score_async(
+                    response=response,
+                    prompt=prompt,
+                    query=query,
+                    context=context,
+                    timeout=self._timeout,
+                    capture_exceptions=True,
+                    batch_index=batch_index,
+                )
+                for batch_index, (response, prompt, query, context) in enumerate(
+                    zip(responses, prompts, queries, contexts)
+                )
+            ]
+        )
+
+        return cast(list[TrustworthyRAGScore], rag_scores)
+
+    async def _batch_async(
+        self,
+        rag_coroutines: Sequence[Coroutine[None, None, Union[TrustworthyRAGResponse, TrustworthyRAGScore]]],
+    ) -> Sequence[Union[TrustworthyRAGResponse, TrustworthyRAGScore]]:
+        """Runs batch of TrustworthyRAG operations.
+
+        Args:
+            rag_coroutines (Sequence[Coroutine[None, None, Union[TrustworthyRAGResponse, TrustworthyRAGScore]]]):
+                list of coroutines to run, returning TrustworthyRAGResponse or TrustworthyRAGScore
+
+        Returns:
+            Sequence[Union[TrustworthyRAGResponse, TrustworthyRAGScore]]: list of coroutine results, with preserved order
+        """
+        rag_query_tasks = [asyncio.create_task(rag_coro) for rag_coro in rag_coroutines]
+
+        if self._verbose:
+            print(
+                "If this progress bar appears frozen, TrustworthyRAG is still processing your dataset so just continue waiting."
+            )
+
+            gather_task = tqdm_asyncio.gather(
+                *rag_query_tasks,
+                total=len(rag_query_tasks),
+                desc="Querying TrustworthyRAG...",
+                bar_format="{desc} {percentage:3.0f}%|{bar}|",
+            )
+        else:
+            gather_task = asyncio.gather(*rag_query_tasks)  # type: ignore[assignment]
+
+        return cast(
+            Sequence[Union[TrustworthyRAGResponse, TrustworthyRAGScore]],
+            await gather_task,
+        )
+
     @handle_tlm_exceptions("TrustworthyRAGResponse")
     async def _generate_async(
         self,
@@ -232,6 +439,7 @@ class TrustworthyRAG(BaseTLM):
         context: str,
         timeout: Optional[float] = None,
         capture_exceptions: bool = False,  # noqa: ARG002
+        batch_index: Optional[int] = None,
     ) -> TrustworthyRAGResponse:
         """
         Private asynchronous method to generate a response and evaluation scores.
@@ -242,7 +450,8 @@ class TrustworthyRAG(BaseTLM):
             query (str): The user's query string that will be evaluated as part of the RAG system.
             context (str): The context/retrieved documents string that will be evaluated as part of the RAG system.
             timeout (float, optional): Timeout (in seconds) for the API call. If None, no timeout is applied.
-            capture_exceptions (bool, optional): Whether to capture exceptions. Not used in the current implementation.
+            capture_exceptions (bool, optional): Whether to capture exceptions rather than propagating them.
+            batch_index (int, optional): Index in the batch for error reporting.
 
         Returns:
             TrustworthyRAGResponse: A [TrustworthyRAGResponse](#class-trustworthyragresponse) object containing
@@ -260,6 +469,8 @@ class TrustworthyRAG(BaseTLM):
                     quality_preset=self._quality_preset,
                     options=self._options,
                     rate_handler=self._rate_handler,
+                    batch_index=batch_index,
+                    retries=_TLM_MAX_RETRIES,
                 ),
                 timeout=timeout,
             ),
@@ -275,6 +486,7 @@ class TrustworthyRAG(BaseTLM):
         context: str,
         timeout: Optional[float] = None,
         capture_exceptions: bool = False,  # noqa: ARG002
+        batch_index: Optional[int] = None,
     ) -> TrustworthyRAGScore:
         """
         Private asynchronous method to obtain evaluation scores for an existing response.
@@ -287,7 +499,8 @@ class TrustworthyRAG(BaseTLM):
             query (str): The user's query string that was used to generate the response.
             context (str): The context/retrieved documents string that was used to generate the response.
             timeout (float, optional): Timeout (in seconds) for the API call. If None, no timeout is applied.
-            capture_exceptions (bool, optional): Whether to capture exceptions. Not used in the current implementation.
+            capture_exceptions (bool, optional): Whether to capture exceptions rather than propagating them.
+            batch_index (int, optional): Index in the batch for error reporting.
 
         Returns:
             TrustworthyRAGScore: A [TrustworthyRAGScore](#class-trustworthyragscore) object containing
@@ -306,6 +519,8 @@ class TrustworthyRAG(BaseTLM):
                     options=self._options,
                     rate_handler=self._rate_handler,
                     evals=self._evals,
+                    batch_index=batch_index,
+                    retries=_TLM_MAX_RETRIES,
                 ),
                 timeout=timeout,
             ),
@@ -364,21 +579,15 @@ class Eval:
     Args:
         name (str): The name of the evaluation, used to identify this specific evaluation in the results.
         criteria (str): The evaluation criteria text that describes what aspect is being evaluated and how.
-        query_identifier (str, optional): Identifier string for the query input in the evaluation criteria.
-            This indicates that the query should be considered when performing this evaluation.
-            The identifier should match what you use in your criteria text.
-            For example, using "User Question" as the identifier means your criteria should refer to the query as "User Question".
-            Set to None if this evaluation doesn't consider the query. Default is None.
-        context_identifier (str, optional): Identifier string for the context input in the evaluation criteria.
-            This indicates that the retrieved context should be considered when performing this evaluation.
-            The identifier should match what you use in your criteria text.
-            For example, using "Retrieved Documents" as the identifier means your criteria should refer to the context as "Retrieved Documents".
-            Set to None if this evaluation doesn't consider the context. Default is None.
-        response_identifier (str, optional): Identifier string for the response input in the evaluation criteria.
-            This indicates that the generated response should be considered when performing this evaluation.
-            The identifier should match what you use in your criteria text.
-            For example, using "AI Answer" as the identifier means your criteria should refer to the response as "AI Answer".
-            Set to None if this evaluation doesn't consider the response. Default is None.
+        query_identifier (str, optional): The exact string used in your evaluation `criteria` to reference the user's query.
+            For example, specifying `query_identifier` as "User Question" means your `criteria` should refer to the query as "User Question".
+            Leave this value as None (the default) if this Eval doesn't consider the query.
+        context_identifier (str, optional): The exact string used in your evaluation `criteria` to reference the retrieved context.
+            For example, specifying `context_identifier` as "Retrieved Documents" means your `criteria` should refer to the context as "Retrieved Documents".
+            Leave this value as None (the default) if this Eval doesn't consider the context.
+        response_identifier (str, optional): The exact string used in your evaluation `criteria` to reference the RAG/LLM response.
+            For example, specifying `response_identifier` as "AI Answer" means your `criteria` should refer to the response as "AI Answer".
+            Leave this value as None (the default) if this Eval doesn't consider the response.
     """
 
     def __init__(
@@ -422,29 +631,37 @@ class Eval:
         )
 
 
-_DEFAULT_EVALS = [
+_DEFAULT_EVALS: list[dict[str, Optional[str]]] = [
     {
         "name": "context_sufficiency",
-        "criteria": "Review the Document and assess whether it contains sufficient information to fully answer the Question. A Document meets the criteria if it provides every essential piece of information needed to construct a complete answer without requiring external knowledge or assumptions. If the Document has gaps that require using general knowledge to fill in, or if it's missing crucial information, it doesn't meet the criteria. Be particularly cautious with Questions that require multiple distinct pieces of information - the more complex the Question, the higher the likelihood that the Document might miss a crucial element. Even a single missing critical piece of information means the Document fails to meet the criteria, as the goal is to ensure the Document alone provides a complete foundation for answering the Question.",
+        "criteria": "Determine if the Document contains 100% of the information needed to answer the Question. If any external knowledge or assumptions are required, it does not meet the criteria. Each Question component must have explicit support in the Document.",
         "query_identifier": "Question",
         "context_identifier": "Document",
         "response_identifier": None,
     },
+    {
+        "name": "response_groundedness",
+        "criteria": "Review the Response to the Query and assess whether every factual claim in the Response is explicitly supported by the provided Context. A Response meets the criteria if all information is directly backed by evidence in the Context, without relying on assumptions, external knowledge, or unstated inferences. The focus is on whether the Response is fully grounded in the Context, rather than whether it fully addresses the Query. If any claim in the Response lacks direct support or introduces information not present in the Context, the Response is bad and does not meet the criteria.",
+        "query_identifier": "Query",
+        "context_identifier": "Context",
+        "response_identifier": "Response",
+    },
     {
         "name": "response_helpfulness",
         "criteria": """Assess whether the AI Assistant Response is a helpful answer to the User Query.
-A Response is not helpful if it:
-- Is not useful, incomplete, or unclear
-- Abstains or refuses to answer the question
-- Contains statements which are similar to 'I don't know', 'Sorry', or 'No information available'
-- Leaves part of the original User Query unresolved""",
+A Response is considered helpful if it makes a genuine attempt to answer the question, even if the answer is incorrect or incomplete. Factual inaccuracies should not affect the assessment. The only thing that matters is whether the Assistant tries to answer the question.
+A Response is considered not helpful if it avoids answering the question. For example, by saying or implying things like "I don't know", "Sorry", "No information available", or any other form of refusal or deflection.""",
         "query_identifier": "User Query",
         "context_identifier": None,
         "response_identifier": "AI Assistant Response",
     },
     {
         "name": "query_ease",
-        "criteria": "Determine whether the above User Request appears simple and straightforward. A bad User Request will appear either: disgruntled, complex, purposefully tricky, abnormal, or vague, perhaps missing vital information needed to answer properly. The simpler the User Request appears, the better. If you believe an AI Assistant could correctly answer this User Request, it is considered good. If the User Request is non-propositional language, it is also considered good.",
+        "criteria": """Determine whether the above User Request appears simple and straightforward.
+A bad User Request will appear either: ambiguous in intent, complex, purposefully tricky, abnormal, or disgruntled.
+A good User Request is phrased clearly and expresses an achievable intent. Basic conversational and non-propositional statements are also considered good.
+Should an AI Assistant be able to properly answer the User Request, it is considered good. The AI Assistant handling this User Request has additional knowledge about: the user, domain-specific terms and abbreviations, and any necessary factual information. So a User Request missing information could still be good; vagueness due to undefined pronouns/terms or references to unknown context does not make a User Request bad.
+""",
         "query_identifier": "User Request",
         "context_identifier": None,
         "response_identifier": None,
@@ -493,7 +710,7 @@ class EvalMetric(TypedDict):
         score (float, optional): score between 0-1 corresponding to the evaluation metric.
         A higher score indicates a higher rating for the specific evaluation criteria being measured.
 
-        log (dict, optional): additional logs and metadata, reported only if the `log` key was specified in `TLMOptions`.
+        log (dict, optional): additional logs and metadata, reported only if the `log` key was specified in [TLMOptions](/tlm/api/python/tlm/#class-tlmoptions).
     """
 
     score: Optional[float]
