@@ -17,6 +17,8 @@ MAX_RESPONSE_LENGTH_TOKENS: int = 15_000
 MAX_COMBINED_LENGTH_TOKENS: int = 70_000
 
 CHARACTERS_PER_TOKEN: int = 4
+# 4 character (3 character + 1 space) = 1 token
+WORD_THAT_EQUALS_ONE_TOKEN = "orb "  # noqa: S105
 
 # Property tests for TLM
 excluded_tlm_models: list[str] = [
@@ -25,13 +27,18 @@ excluded_tlm_models: list[str] = [
     "claude-3.5-sonnet",
     "claude-3.5-sonnet-v2",
     "claude-3.7-sonnet",
+    "claude-opus-4-0",
+    "claude-sonnet-4-0",
     "o1-preview",
     "o1",
     "o1-mini",
+    "o3",
     "o3-mini",
+    "o4-mini",
     "nova-lite",
     "nova-pro",
     "gpt-4",
+    "gpt-4.1",
     "gpt-4.5-preview",
 ]
 VALID_TLM_MODELS: list[str] = [model for model in _VALID_TLM_MODELS if model not in excluded_tlm_models]
@@ -42,10 +49,14 @@ MODELS_WITH_NO_PERPLEXITY_SCORE: list[str] = [
     "claude-3.5-sonnet",
     "claude-3.5-sonnet-v2",
     "claude-3.7-sonnet",
+    "claude-opus-4-0",
+    "claude-sonnet-4-0",
     "o1-preview",
     "o1",
     "o1-mini",
+    "o3",
     "o3-mini",
+    "o4-mini",
     "gpt-4.5-preview",
     "nova-micro",
     "nova-lite",
