@@ -1,12 +1,16 @@
 import os
+from collections.abc import Generator
 from typing import Any, cast
 from unittest import mock
 
 import pytest
 
-from cleanlab_tlm.errors import MissingApiKeyError, ValidationError
+from cleanlab_tlm.errors import APITimeoutError, MissingApiKeyError, ValidationError
 from cleanlab_tlm.internal.api import api
-from cleanlab_tlm.internal.constants import _TLM_DEFAULT_MODEL
+from cleanlab_tlm.internal.constants import (
+    _TLM_DEFAULT_MODEL,
+    _VALID_TLM_QUALITY_PRESETS,
+)
 from cleanlab_tlm.tlm import TLMOptions
 from cleanlab_tlm.utils.rag import (
     _DEFAULT_EVALS,
@@ -31,6 +35,26 @@ test_context = make_text_unique(TEST_CONTEXT)
 test_response = make_text_unique(TEST_RESPONSE)
 test_prompt = make_text_unique(TEST_PROMPT)
 
+# Create test batches
+test_query_batch = [
+    make_text_unique("What is the capital of France?"),
+    make_text_unique("What is the capital of Germany?"),
+]
+test_context_batch = [
+    make_text_unique(
+        "France is a country in Western Europe. Its capital is Paris, which is known for the Eiffel Tower."
+    ),
+    make_text_unique("Germany is a country in Central Europe. Its capital is Berlin, known for the Brandenburg Gate."),
+]
+test_response_batch = [
+    make_text_unique("The capital of France is Paris."),
+    make_text_unique("The capital of Germany is Berlin."),
+]
+test_prompt_batch = [
+    make_text_unique("Using the context information, answer the following question: What is the capital of France?"),
+    make_text_unique("Using the context information, answer the following question: What is the capital of Germany?"),
+]
+
 
 @pytest.fixture(scope="module")
 def trustworthy_rag_api_key() -> str:
@@ -158,7 +182,7 @@ def test_init_with_options(trustworthy_rag_api_key: str) -> None:
     options: TLMOptions = {
         "model": "gpt-4",
         "max_tokens": max_tokens,
-        "use_self_reflection": True,
+        "num_self_reflections": 3,
     }
 
     rag = TrustworthyRAG(api_key=trustworthy_rag_api_key, options=options)
@@ -167,20 +191,14 @@ def test_init_with_options(trustworthy_rag_api_key: str) -> None:
     assert rag._options is not None
     assert rag._options["model"] == "gpt-4"
     assert rag._options["max_tokens"] == max_tokens
-    assert rag._options["use_self_reflection"] is True
+    assert rag._options["num_self_reflections"] == 3  # noqa: PLR2004
 
 
-def test_init_with_quality_preset(trustworthy_rag_api_key: str) -> None:
-    rag_medium = TrustworthyRAG(quality_preset="medium", api_key=trustworthy_rag_api_key)
-    assert rag_medium is not None
-    assert rag_medium._quality_preset == "medium"
-
-    rag_low = TrustworthyRAG(quality_preset="low", api_key=trustworthy_rag_api_key)
-    assert rag_low is not None
-    assert rag_low._quality_preset == "low"
-
-    with pytest.raises(ValidationError):
-        TrustworthyRAG(quality_preset="high", api_key=trustworthy_rag_api_key)
+@pytest.mark.parametrize("quality_preset", _VALID_TLM_QUALITY_PRESETS)
+def test_init_with_quality_preset(trustworthy_rag_api_key: str, quality_preset: str) -> None:
+    tlm_rag = TrustworthyRAG(quality_preset=quality_preset, api_key=trustworthy_rag_api_key)  # type: ignore
+    assert tlm_rag is not None
+    assert tlm_rag._quality_preset == quality_preset
 
 
 def test_get_evals(trustworthy_rag: TrustworthyRAG) -> None:
@@ -765,3 +783,175 @@ def test_init_with_unsupported_quality_preset(trustworthy_rag_api_key: str) -> N
             quality_preset="unsupported_preset",  # type: ignore
             api_key=trustworthy_rag_api_key,
         )
+
+
+def test_batch_generate(trustworthy_rag: TrustworthyRAG) -> None:
+    """Tests batch generate functionality of TrustworthyRAG.
+
+    Expected:
+    - TrustworthyRAG should return a list of TrustworthyRAGResponse objects
+    - Each response should be a valid TrustworthyRAGResponse object
+    """
+    # act
+    responses = trustworthy_rag.generate(
+        query=test_query_batch,
+        context=test_context_batch,
+    )
+
+    # assert
+    assert isinstance(responses, list)
+    assert all(is_trustworthy_rag_response(r) for r in responses)
+
+
+def test_batch_score(trustworthy_rag: TrustworthyRAG) -> None:
+    """Tests batch score functionality of TrustworthyRAG.
+
+    Expected:
+    - TrustworthyRAG should return a list of TrustworthyRAGScore objects
+    - Each score should be a valid TrustworthyRAGScore object
+    """
+    # act
+    scores = trustworthy_rag.score(
+        query=test_query_batch,
+        context=test_context_batch,
+        response=test_response_batch,
+    )
+
+    # assert
+    assert isinstance(scores, list)
+    assert all(is_trustworthy_rag_score(s) for s in scores)
+
+
+def test_batch_generate_with_prompt(trustworthy_rag: TrustworthyRAG) -> None:
+    """Tests batch generate functionality of TrustworthyRAG with prompt.
+
+    Expected:
+    - TrustworthyRAG should return a list of TrustworthyRAGResponse objects
+    - Each response should be a valid TrustworthyRAGResponse object
+    """
+    # act
+    responses = trustworthy_rag.generate(
+        query=test_query_batch,
+        context=test_context_batch,
+        prompt=test_prompt_batch,
+    )
+
+    # assert
+    assert isinstance(responses, list)
+    assert all(is_trustworthy_rag_response(r) for r in responses)
+
+
+def test_batch_score_with_prompt(trustworthy_rag: TrustworthyRAG) -> None:
+    """Tests batch score functionality of TrustworthyRAG with prompt.
+
+    Expected:
+    - TrustworthyRAG should return a list of TrustworthyRAGScore objects
+    - Each score should be a valid TrustworthyRAGScore object
+    """
+    # act
+    scores = trustworthy_rag.score(
+        query=test_query_batch,
+        context=test_context_batch,
+        response=test_response_batch,
+        prompt=test_prompt_batch,
+    )
+
+    # assert
+    assert isinstance(scores, list)
+    assert all(is_trustworthy_rag_score(s) for s in scores)
+
+
+def test_batch_generate_with_custom_form_prompt(trustworthy_rag: TrustworthyRAG) -> None:
+    """Tests batch generate functionality of TrustworthyRAG with custom form_prompt.
+
+    Expected:
+    - TrustworthyRAG should return a list of TrustworthyRAGResponse objects
+    - Each response should be a valid TrustworthyRAGResponse object
+    """
+
+    def custom_form_prompt(query: str, context: str) -> str:
+        return f"System: Always be helpful\nContext: {context}\nUser: {query}"
+
+    # act
+    responses = trustworthy_rag.generate(
+        query=test_query_batch,
+        context=test_context_batch,
+        form_prompt=custom_form_prompt,
+    )
+
+    # assert
+    assert isinstance(responses, list)
+    assert all(is_trustworthy_rag_response(r) for r in responses)
+
+
+def test_batch_score_with_custom_form_prompt(trustworthy_rag: TrustworthyRAG) -> None:
+    """Tests batch score functionality of TrustworthyRAG with custom form_prompt.
+
+    Expected:
+    - TrustworthyRAG should return a list of TrustworthyRAGScore objects
+    - Each score should be a valid TrustworthyRAGScore object
+    """
+
+    def custom_form_prompt(query: str, context: str) -> str:
+        return f"System: Always be helpful\nContext: {context}\nUser: {query}"
+
+    # act
+    scores = trustworthy_rag.score(
+        query=test_query_batch,
+        context=test_context_batch,
+        response=test_response_batch,
+        form_prompt=custom_form_prompt,
+    )
+
+    # assert
+    assert isinstance(scores, list)
+    assert all(is_trustworthy_rag_score(s) for s in scores)
+
+
+def test_generate_force_timeouts(trustworthy_rag: TrustworthyRAG, reset_rag_timeout: None) -> None:  # noqa: ARG001
+    """Tests single prompt generate with forced timeouts.
+
+    Sets timeout to 0.0001 seconds, which should force a timeout error being thrown.
+
+    Expected:
+    - TrustworthyRAG should raise a timeout error
+    """
+    # arrange -- override timeout
+    trustworthy_rag._timeout = 0.0001
+
+    # assert -- timeout is thrown
+    with pytest.raises(APITimeoutError):
+        # act -- run a generate
+        trustworthy_rag.generate(
+            query=test_query,
+            context=test_context,
+        )
+
+
+def test_score_force_timeouts(trustworthy_rag: TrustworthyRAG, reset_rag_timeout: None) -> None:  # noqa: ARG001
+    """Tests single score with forced timeouts.
+
+    Sets timeout to 0.0001 seconds, which should force a timeout error being thrown.
+
+    Expected:
+    - TrustworthyRAG should raise a timeout error
+    """
+    # arrange -- override timeout
+    trustworthy_rag._timeout = 0.0001
+
+    # assert -- timeout is thrown
+    with pytest.raises(APITimeoutError):
+        # act -- run a batch score
+        trustworthy_rag.score(
+            query=test_query,
+            context=test_context,
+            response=test_response,
+        )
+
+
+@pytest.fixture
+def reset_rag_timeout(trustworthy_rag: TrustworthyRAG) -> Generator[None, None, None]:
+    """Reset the timeout on the TrustworthyRAG fixture after tests that modify it."""
+    old_timeout = trustworthy_rag._timeout
+    yield
+    trustworthy_rag._timeout = old_timeout
