@@ -1,4 +1,4 @@
-from typing import Any
+from typing import Any, cast
 
 import numpy as np
 import pytest
@@ -8,16 +8,17 @@ from cleanlab_tlm.internal.constants import (
     _VALID_TLM_TASKS,
     TLM_TASK_SUPPORTING_CONSTRAIN_OUTPUTS,
 )
-from cleanlab_tlm.tlm import TLM, TLMOptions
+from cleanlab_tlm.tlm import TLM, TLMOptions, TLMResponse, TLMScore
+from cleanlab_tlm.utils.rag import Eval, TrustworthyRAG
 from tests.conftest import make_text_unique
 from tests.constants import (
-    CHARACTERS_PER_TOKEN,
     MAX_COMBINED_LENGTH_TOKENS,
     MAX_PROMPT_LENGTH_TOKENS,
     MAX_RESPONSE_LENGTH_TOKENS,
     TEST_PROMPT,
     TEST_PROMPT_BATCH,
     TEST_RESPONSE,
+    WORD_THAT_EQUALS_ONE_TOKEN,
 )
 from tests.test_get_trustworthiness_score import is_tlm_score_response_with_error
 from tests.test_prompt import is_tlm_response_with_error
@@ -207,7 +208,7 @@ def test_prompt_too_long_exception_single_prompt(tlm: TLM) -> None:
     """Tests that bad request error is raised when prompt is too long when calling tlm.prompt with a single prompt."""
     with pytest.raises(TlmBadRequestError) as exc_info:
         tlm.prompt(
-            "a" * (MAX_PROMPT_LENGTH_TOKENS + 1) * CHARACTERS_PER_TOKEN,
+            WORD_THAT_EQUALS_ONE_TOKEN * (MAX_PROMPT_LENGTH_TOKENS + 1),
         )
 
     assert exc_info.value.message.startswith("Prompt length exceeds")
@@ -215,35 +216,14 @@ def test_prompt_too_long_exception_single_prompt(tlm: TLM) -> None:
 
 
 @pytest.mark.parametrize("num_prompts", [1, 2, 5])
-def test_prompt_too_long_exception_batch_prompt(tlm: TLM, num_prompts: int) -> None:
-    """Tests that bad request error is raised when prompt is too long when calling tlm.prompt with a batch of prompts.
-
-    Error message should indicate which the batch index for which the prompt is too long.
-    """
+def test_prompt_too_long_exception_prompt(tlm: TLM, num_prompts: int) -> None:
+    """Tests that None is returned when prompt is too long when calling tlm.prompt with a batch of prompts."""
     # create batch of prompts with one prompt that is too long
     prompts = [test_prompt] * num_prompts
     prompt_too_long_index = np.random.randint(0, num_prompts)
-    prompts[prompt_too_long_index] = "a" * (MAX_PROMPT_LENGTH_TOKENS + 1) * CHARACTERS_PER_TOKEN
+    prompts[prompt_too_long_index] = WORD_THAT_EQUALS_ONE_TOKEN * (MAX_PROMPT_LENGTH_TOKENS + 1)
 
-    with pytest.raises(TlmBadRequestError) as exc_info:
-        tlm.prompt(prompts)
-
-    assert exc_info.value.message.startswith(f"Error executing query at index {prompt_too_long_index}:")
-    assert "Prompt length exceeds" in exc_info.value.message
-    assert exc_info.value.retryable is False
-
-
-@pytest.mark.parametrize("num_prompts", [1, 2, 5])
-def test_prompt_too_long_exception_try_prompt(tlm: TLM, num_prompts: int) -> None:
-    """Tests that None is returned when prompt is too long when calling tlm.try_prompt with a batch of prompts."""
-    # create batch of prompts with one prompt that is too long
-    prompts = [test_prompt] * num_prompts
-    prompt_too_long_index = np.random.randint(0, num_prompts)
-    prompts[prompt_too_long_index] = "a" * (MAX_PROMPT_LENGTH_TOKENS + 1) * CHARACTERS_PER_TOKEN
-
-    tlm_responses = tlm.try_prompt(
-        prompts,
-    )
+    tlm_responses = cast(list[TLMResponse], tlm.prompt(prompts))
 
     assert_prompt_too_long_error(tlm_responses[prompt_too_long_index], prompt_too_long_index)
 
@@ -252,8 +232,8 @@ def test_response_too_long_exception_single_score(tlm: TLM) -> None:
     """Tests that bad request error is raised when response is too long when calling tlm.get_trustworthiness_score with a single prompt."""
     with pytest.raises(TlmBadRequestError) as exc_info:
         tlm.get_trustworthiness_score(
-            "a",
-            "a" * (MAX_RESPONSE_LENGTH_TOKENS + 1) * CHARACTERS_PER_TOKEN,
+            WORD_THAT_EQUALS_ONE_TOKEN,
+            WORD_THAT_EQUALS_ONE_TOKEN * (MAX_RESPONSE_LENGTH_TOKENS + 1),
         )
 
     assert exc_info.value.message.startswith("Response length exceeds")
@@ -261,41 +241,15 @@ def test_response_too_long_exception_single_score(tlm: TLM) -> None:
 
 
 @pytest.mark.parametrize("num_prompts", [1, 2, 5])
-def test_response_too_long_exception_batch_score(tlm: TLM, num_prompts: int) -> None:
-    """Tests that bad request error is raised when prompt is too long when calling tlm.get_trustworthiness_score with a batch of prompts.
-
-    Error message should indicate which the batch index for which the prompt is too long.
-    """
+def test_response_too_long_exception_score(tlm: TLM, num_prompts: int) -> None:
+    """Tests that None is returned when prompt is too long when calling tlm.get_trustworthiness_score with a batch of prompts."""
     # create batch of prompts with one prompt that is too long
     prompts = [test_prompt] * num_prompts
     responses = [TEST_RESPONSE] * num_prompts
     response_too_long_index = np.random.randint(0, num_prompts)
-    responses[response_too_long_index] = "a" * (MAX_RESPONSE_LENGTH_TOKENS + 1) * CHARACTERS_PER_TOKEN
+    responses[response_too_long_index] = WORD_THAT_EQUALS_ONE_TOKEN * (MAX_RESPONSE_LENGTH_TOKENS + 1)
 
-    with pytest.raises(TlmBadRequestError) as exc_info:
-        tlm.get_trustworthiness_score(
-            prompts,
-            responses,
-        )
-
-    assert exc_info.value.message.startswith(f"Error executing query at index {response_too_long_index}:")
-    assert "Response length exceeds" in exc_info.value.message
-    assert exc_info.value.retryable is False
-
-
-@pytest.mark.parametrize("num_prompts", [1, 2, 5])
-def test_response_too_long_exception_try_score(tlm: TLM, num_prompts: int) -> None:
-    """Tests that None is returned when prompt is too long when calling tlm.try_get_trustworthiness_score with a batch of prompts."""
-    # create batch of prompts with one prompt that is too long
-    prompts = [test_prompt] * num_prompts
-    responses = [TEST_RESPONSE] * num_prompts
-    response_too_long_index = np.random.randint(0, num_prompts)
-    responses[response_too_long_index] = "a" * (MAX_RESPONSE_LENGTH_TOKENS + 1) * CHARACTERS_PER_TOKEN
-
-    tlm_responses = tlm.try_get_trustworthiness_score(
-        prompts,
-        responses,
-    )
+    tlm_responses = cast(list[TLMScore], tlm.get_trustworthiness_score(prompts, responses))
 
     assert_response_too_long_error_score(tlm_responses[response_too_long_index], response_too_long_index)
 
@@ -304,8 +258,8 @@ def test_prompt_too_long_exception_single_score(tlm: TLM) -> None:
     """Tests that bad request error is raised when prompt is too long when calling tlm.get_trustworthiness_score with a single prompt."""
     with pytest.raises(TlmBadRequestError) as exc_info:
         tlm.get_trustworthiness_score(
-            "a" * (MAX_PROMPT_LENGTH_TOKENS + 1) * CHARACTERS_PER_TOKEN,
-            "a",
+            WORD_THAT_EQUALS_ONE_TOKEN * (MAX_PROMPT_LENGTH_TOKENS + 1),
+            WORD_THAT_EQUALS_ONE_TOKEN,
         )
 
     assert exc_info.value.message.startswith("Prompt length exceeds")
@@ -313,41 +267,15 @@ def test_prompt_too_long_exception_single_score(tlm: TLM) -> None:
 
 
 @pytest.mark.parametrize("num_prompts", [1, 2, 5])
-def test_prompt_too_long_exception_batch_score(tlm: TLM, num_prompts: int) -> None:
-    """Tests that bad request error is raised when prompt is too long when calling tlm.get_trustworthiness_score with a batch of prompts.
-
-    Error message should indicate which the batch index for which the prompt is too long.
-    """
+def test_prompt_too_long_exception_score(tlm: TLM, num_prompts: int) -> None:
+    """Tests that None is returned when prompt is too long when calling tlm.get_trustworthiness_score with a batch of prompts."""
     # create batch of prompts with one prompt that is too long
     prompts = [test_prompt] * num_prompts
     responses = [TEST_RESPONSE] * num_prompts
     prompt_too_long_index = np.random.randint(0, num_prompts)
-    prompts[prompt_too_long_index] = "a" * (MAX_PROMPT_LENGTH_TOKENS + 1) * CHARACTERS_PER_TOKEN
+    prompts[prompt_too_long_index] = WORD_THAT_EQUALS_ONE_TOKEN * (MAX_PROMPT_LENGTH_TOKENS + 1)
 
-    with pytest.raises(TlmBadRequestError) as exc_info:
-        tlm.get_trustworthiness_score(
-            prompts,
-            responses,
-        )
-
-    assert exc_info.value.message.startswith(f"Error executing query at index {prompt_too_long_index}:")
-    assert "Prompt length exceeds" in exc_info.value.message
-    assert exc_info.value.retryable is False
-
-
-@pytest.mark.parametrize("num_prompts", [1, 2, 5])
-def test_prompt_too_long_exception_try_score(tlm: TLM, num_prompts: int) -> None:
-    """Tests that None is returned when prompt is too long when calling tlm.try_get_trustworthiness_score with a batch of prompts."""
-    # create batch of prompts with one prompt that is too long
-    prompts = [test_prompt] * num_prompts
-    responses = [TEST_RESPONSE] * num_prompts
-    prompt_too_long_index = np.random.randint(0, num_prompts)
-    prompts[prompt_too_long_index] = "a" * (MAX_PROMPT_LENGTH_TOKENS + 1) * CHARACTERS_PER_TOKEN
-
-    tlm_responses = tlm.try_get_trustworthiness_score(
-        prompts,
-        responses,
-    )
+    tlm_responses = cast(list[TLMScore], tlm.get_trustworthiness_score(prompts, responses))
 
     assert_prompt_too_long_error_score(tlm_responses[prompt_too_long_index], prompt_too_long_index)
 
@@ -358,8 +286,8 @@ def test_combined_too_long_exception_single_score(tlm: TLM) -> None:
 
     with pytest.raises(TlmBadRequestError) as exc_info:
         tlm.get_trustworthiness_score(
-            "a" * max_prompt_length * CHARACTERS_PER_TOKEN,
-            "a" * MAX_RESPONSE_LENGTH_TOKENS * CHARACTERS_PER_TOKEN,
+            WORD_THAT_EQUALS_ONE_TOKEN * max_prompt_length,
+            WORD_THAT_EQUALS_ONE_TOKEN * MAX_RESPONSE_LENGTH_TOKENS,
         )
 
     assert exc_info.value.message.startswith("Prompt and response combined length exceeds")
@@ -378,34 +306,10 @@ def test_prompt_and_response_combined_too_long_exception_batch_score(tlm: TLM, n
     combined_too_long_index = np.random.randint(0, num_prompts)
 
     max_prompt_length = MAX_COMBINED_LENGTH_TOKENS - MAX_RESPONSE_LENGTH_TOKENS + 1
-    prompts[combined_too_long_index] = "a" * max_prompt_length * CHARACTERS_PER_TOKEN
-    responses[combined_too_long_index] = "a" * MAX_RESPONSE_LENGTH_TOKENS * CHARACTERS_PER_TOKEN
+    prompts[combined_too_long_index] = WORD_THAT_EQUALS_ONE_TOKEN * max_prompt_length
+    responses[combined_too_long_index] = WORD_THAT_EQUALS_ONE_TOKEN * MAX_RESPONSE_LENGTH_TOKENS
 
-    tlm_responses = tlm.try_get_trustworthiness_score(
-        prompts,
-        responses,
-    )
-
-    assert_prompt_and_response_combined_too_long_error_score(
-        tlm_responses[combined_too_long_index], combined_too_long_index
-    )
-
-
-@pytest.mark.parametrize("num_prompts", [1, 2, 5])
-def test_prompt_and_response_combined_too_long_exception_try_score(tlm: TLM, num_prompts: int) -> None:
-    """Tests that appropriate error is returned when prompt + response is too long when calling tlm.try_get_trustworthiness_score with a batch of prompts."""
-    # create batch of prompts with one prompt that is too long
-    prompts = [test_prompt] * num_prompts
-    responses = [TEST_RESPONSE] * num_prompts
-    combined_too_long_index = np.random.randint(0, num_prompts)
-    max_prompt_length = MAX_COMBINED_LENGTH_TOKENS - MAX_RESPONSE_LENGTH_TOKENS + 1
-    prompts[combined_too_long_index] = "a" * max_prompt_length * CHARACTERS_PER_TOKEN
-    responses[combined_too_long_index] = "a" * MAX_RESPONSE_LENGTH_TOKENS * CHARACTERS_PER_TOKEN
-
-    tlm_responses = tlm.try_get_trustworthiness_score(
-        prompts,
-        responses,
-    )
+    tlm_responses = cast(list[TLMScore], tlm.get_trustworthiness_score(prompts, responses))
 
     assert_prompt_and_response_combined_too_long_error_score(
         tlm_responses[combined_too_long_index], combined_too_long_index
@@ -451,16 +355,6 @@ def test_validate_rag_inputs_prompt_and_form_prompt_together() -> None:
     assert "prompt' and 'form_prompt' cannot be provided at the same time" in str(exc_info.value)
 
 
-def test_validate_rag_inputs_batch_not_supported() -> None:
-    """Tests that NotImplementedError is raised when batch inputs are provided."""
-    from cleanlab_tlm.internal.validation import validate_rag_inputs
-
-    with pytest.raises(NotImplementedError) as exc_info:
-        validate_rag_inputs(query=["query1", "query2"], context="test context", is_generate=True)
-
-    assert "Batch processing is not yet supported" in str(exc_info.value)
-
-
 def test_validate_rag_inputs_generate_missing_required_params() -> None:
     """Tests that ValidationError is raised when required parameters are missing for generate."""
     from cleanlab_tlm.internal.validation import validate_rag_inputs
@@ -492,7 +386,7 @@ def test_validate_rag_inputs_score_missing_required_params() -> None:
     with pytest.raises(ValidationError) as exc_info:
         validate_rag_inputs(query=None, context=None, response="test response", is_generate=False)  # type: ignore
 
-    assert "Either 'prompt' or both 'query' and 'context' must be provided" in str(exc_info.value)
+    assert "Both 'query' and 'context' are required parameters" in str(exc_info.value)
 
 
 def test_validate_rag_inputs_form_prompt_missing_params() -> None:
@@ -513,14 +407,99 @@ def test_validate_rag_inputs_invalid_param_types() -> None:
     """Tests that ValidationError is raised when parameters have invalid types."""
     from cleanlab_tlm.internal.validation import validate_rag_inputs
 
+    # Test invalid query type
     with pytest.raises(ValidationError) as exc_info:
         validate_rag_inputs(
             query=123,  # type: ignore
             context="test context",
             is_generate=True,
         )
+    assert "'query' must be either a string or a sequence of strings, not <class 'int'>" in str(exc_info.value)
 
-    assert "'query' must be a string" in str(exc_info.value)
+    # Test invalid context type
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            query="test query",
+            context=456,  # type: ignore
+            is_generate=True,
+        )
+    assert "'context' must be either a string or a sequence of strings, not <class 'int'>" in str(exc_info.value)
+
+    # Test invalid response type
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            query="test query",
+            context="test context",
+            response=789,  # type: ignore
+            is_generate=False,
+        )
+    assert "'response' must be either a string or a sequence of strings, not <class 'int'>" in str(exc_info.value)
+
+    # Test invalid prompt type
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            query="test query",
+            context="test context",
+            prompt=True,  # type: ignore
+            is_generate=True,
+        )
+    assert "'prompt' must be either a string or a sequence of strings, not <class 'bool'>" in str(exc_info.value)
+
+    # Test sequence with non-string items
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            query=["valid", 123, "also valid"],  # type: ignore
+            context=["test context1", "test context2", "test context3"],
+            is_generate=True,
+        )
+    assert "All items in 'query' must be of type string when providing a sequence" in str(exc_info.value)
+
+    # Test mismatched length of query and context sequences with a proper form_prompt function
+    def valid_form_prompt(q: str, c: str) -> str:
+        return f"{q} {c}"
+
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            query=["q1", "q2", "q3"],
+            context=["c1", "c2"],
+            form_prompt=valid_form_prompt,
+            is_generate=True,
+        )
+    assert (
+        "Input lists have different lengths: query: 3, context: 2. All input lists must have the same length."
+        in str(exc_info.value)
+    )
+
+
+def test_validate_proper_evals_input(tlm_api_key: str) -> None:
+    evals = [
+        Eval(
+            name="test_eval",
+            criteria="This is a test criteria",
+            query_identifier="query",
+            context_identifier="context",
+            response_identifier="response",
+        )
+    ]
+
+    # test the expected case will work
+    tlm_rag = TrustworthyRAG(api_key=tlm_api_key, evals=evals)
+    assert tlm_rag is not None
+
+    # test passing a list of list of evals
+    with pytest.raises(ValidationError) as exc_info:
+        tlm_rag = TrustworthyRAG(api_key=tlm_api_key, evals=[evals])  # type: ignore
+    assert "'evals' must be a list of Eval objects" in str(exc_info.value)
+
+    # test passing a list of dicts
+    with pytest.raises(ValidationError) as exc_info:
+        tlm_rag = TrustworthyRAG(api_key=tlm_api_key, evals=[{"test_eval": "This is a test criteria"}])  # type: ignore
+    assert "'evals' must be a list of Eval objects" in str(exc_info.value)
+
+    # test passing a string
+    with pytest.raises(ValidationError) as exc_info:
+        tlm_rag = TrustworthyRAG(api_key=tlm_api_key, evals="This is a test criteria")  # type: ignore
+    assert "'evals' must be a list of Eval objects" in str(exc_info.value)
 
 
 def test_validate_rag_inputs_with_evals() -> None:
@@ -645,7 +624,7 @@ def test_custom_eval_criteria_validation(tlm_api_key: str) -> None:
     # Invalid: missing name
     with pytest.raises(
         ValidationError,
-        match="^Missing required key 'name' in custom_eval_criteria item 0.$",
+        match="^Missing required keys {'name'} in custom_eval_criteria item 0.$",
     ):
         TLM(
             api_key=tlm_api_key,
@@ -655,13 +634,23 @@ def test_custom_eval_criteria_validation(tlm_api_key: str) -> None:
     # Invalid: missing criteria
     with pytest.raises(
         ValidationError,
-        match="^Missing required key 'criteria' in custom_eval_criteria item 0.$",
+        match="^Missing required keys {'criteria'} in custom_eval_criteria item 0.$",
     ):
         TLM(
             api_key=tlm_api_key,
             options=TLMOptions(custom_eval_criteria=[{"name": "test"}]),
         )
 
+    # Invalid: extra keys
+    with pytest.raises(
+        ValidationError,
+        match="^Invalid keys {'extra'} found in custom_eval_criteria item 0. Supported keys are: ({'name', 'criteria'}|{'criteria', 'name'}).$",
+    ):
+        TLM(
+            api_key=tlm_api_key,
+            options=TLMOptions(custom_eval_criteria=[{"name": "test", "criteria": "test", "extra": "extra"}]),
+        )
+
     # Invalid: name not a string
     with pytest.raises(
         ValidationError,
@@ -697,3 +686,115 @@ def test_validate_tlm_options_support_custom_eval_criteria() -> None:
         match="^custom_eval_criteria is not supported for this class$",
     ):
         validate_tlm_options(options, support_custom_eval_criteria=False)
+
+
+def test_validate_rag_inputs_mixed_string_and_sequence() -> None:
+    """Tests that validate_rag_inputs rejects mixed inputs where some are strings and others are sequences."""
+    from cleanlab_tlm.internal.validation import validate_rag_inputs
+
+    # Test string query with sequence context (for generate)
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            query="test query",
+            context=["context 1", "context 2"],
+            is_generate=True,
+        )
+
+    assert "Inconsistent input formats" in str(exc_info.value)
+    assert "query is a string while other inputs are lists" in str(exc_info.value)
+
+    # Test with prompt as a sequence but query and context as strings (for score)
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            prompt=["prompt 1", "prompt 2", "prompt 3"],
+            query="test query",
+            context="test context",
+            response="test response",
+            is_generate=False,
+        )
+
+    assert "Inconsistent input formats" in str(exc_info.value)
+
+    # Test with response as a sequence but query and context as strings (for score)
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            prompt="test prompt",
+            query="test query",
+            context="test context",
+            response=["response 1", "response 2"],
+            is_generate=False,
+        )
+
+    assert "Inconsistent input formats" in str(exc_info.value)
+
+    # Test the specific case provided by the user
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            prompt=["sample"],
+            response=["sample"],
+            query="sample",
+            context="sample",
+            is_generate=False,
+        )
+
+    assert "Inconsistent input formats" in str(exc_info.value)
+
+
+def test_validate_rag_inputs_list_length_mismatch() -> None:
+    """Tests that validate_rag_inputs rejects lists with different lengths."""
+    from cleanlab_tlm.internal.validation import validate_rag_inputs
+
+    # Test lists with different lengths (for generate)
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            query=["query 1", "query 2", "query 3"],
+            context=["context 1", "context 2"],
+            is_generate=True,
+        )
+
+    assert "Input lists have different lengths" in str(exc_info.value)
+
+    # Test lists with different lengths (for score)
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            query=["query 1", "query 2"],
+            context=["context 1", "context 2"],
+            response=["response 1", "response 2", "response 3"],
+            is_generate=False,
+        )
+
+    assert "Input lists have different lengths" in str(exc_info.value)
+
+    # Test lists with different lengths including prompt (for score)
+    with pytest.raises(ValidationError) as exc_info:
+        validate_rag_inputs(
+            prompt=["prompt 1", "prompt 2", "prompt 3"],
+            query=["query 1", "query 2"],
+            context=["context 1", "context 2"],
+            response=["response 1", "response 2"],
+            is_generate=False,
+        )
+
+    assert "Input lists have different lengths" in str(exc_info.value)
+
+
+def test_validate_rag_inputs_matching_lists() -> None:
+    """Tests that validate_rag_inputs accepts lists with matching lengths."""
+    from cleanlab_tlm.internal.validation import validate_rag_inputs
+
+    list_length = 2
+
+    # Test with matching list lengths and form_prompt (for score)
+    result = validate_rag_inputs(
+        query=["query 1", "query 2"],
+        context=["context 1", "context 2"],
+        response=["response 1", "response 2"],
+        form_prompt=lambda q, c: f"Q: {q} C: {c}",
+        is_generate=False,
+    )
+
+    # Should get a list of formatted prompts
+    assert isinstance(result, list)
+    assert len(result) == list_length
+    assert result[0] == "Q: query 1 C: context 1"
+    assert result[1] == "Q: query 2 C: context 2"
